{-# LANGUAGE DeriveDataTypeable #-}

module Abbrevs where 

import Tct.Interactive
import Tct.Instances
import qualified Termlib.Repl as TR
import Data.Typeable
import Tct.Processor.Args.Instances
import Tct.Processor.Args

data BaseTechniques = Polys | Matrices | PopStar deriving (Typeable, Show, Enum, Bounded, Eq)      

directsArg :: Arg [EnumOf BaseTechniques]
directsArg = arg 


whenNonTrivial t = withProblem $ \ prob ->
  when (not $ TR.isEmpty $ TR.strictComponents prob) t
  
whenStrictRules t = 
  withProblem $ \ prob -> 
  when (not $ TR.isEmpty $ TR.strictTrs prob) t

wg dim deg = weightgap 
             `withCertBy` cert' 
             `withDimension` dim' 
             `withDegree` deg' 
             `withCBits` Just (bits' + 1) 
             `withBits` bits'
             `wgOn` WgOnAny
  where bits' | dim <= 3  = 2
              | otherwise = 1
        dim' = max 1 dim
        deg' = if dim' <= deg then Nothing else Just (max 0 deg)
        cert' | deg' == Just 0 = Algebraic
              | otherwise = Automaton

mx dim deg = matrix 
             `withCertBy` cert' 
             `withDimension` dim' 
             `withDegree` deg' 
             `withCBits` Just (bits' + 1) 
             `withBits` bits'
  where bits' | dim <= 2  = 2
              | otherwise = 1
        dim' = max 1 dim
        deg' = if dim' <= deg then Nothing else Just (max 0 deg)
        cert' | deg' == Just 0 = Algebraic
              | otherwise = Automaton
                      
matchbounds = bounds Minimal Match `orFaster` bounds PerSymbol Match

matrices i = fastest [ mx i i, mx (i + 2) i]

simps = 
  te removeHeads
  >>> try cleanSuffix
  >>> try trivial
  >>> try usableRules 

basics = 
  timeout 5 matchbounds 
  `orFaster` (te (trivial <||> (sems >>> try simps)) >>! popstars)
  where
    sems =  
      pe (poly `withDegree` Just 3)
      <||> pe (mx 3 3)
      <||> pe (mx 4 3)                        
      <||> pe (mx 4 4)
    popstars = popstarPS `before` popstar
    pe proc = proc `withPEOn` selAnyOf selStricts

                   
cleanUpTail = 
  (removeWeakSuffix >>> try simpDPRHS)
  >>> try usableRules
  >>> try trivial  
                   
shiftSemanticBy cmp 0 = some $ whenNonTrivial $ cmp (some $ mx 1 0)
shiftSemanticBy cmp i = some $ 
  whenNonTrivial $ 
  cmp (some $ mx i i)
  <||> cmp (some $ mx (i+1) i)
  <||> cmp (some $ spopstarPS `withDegree` Just i)
  <||> when (i == 2 || i == 3) (cmp (some $ poly `withDegree` Just i))

shiftRulesSuccessively = force (successive [te $ whenStrictRules $ shift i | i <- [1..4]]) >>> try dpsimps
  where selAnyStrictRules = selAnyOf $ selRules `selInter` selStricts
        shift i = shiftSemanticBy (decompose selAnyStrictRules Add) i
